# chloro - personal space
# Copyright (C) 2019  Frederic Peters
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.core.exceptions import PermissionDenied
from django.http import Http404
from django.views.generic import CreateView, DeleteView, DetailView, ListView, UpdateView, TemplateView

from .models import Note


class NoteView(DetailView):
    model = Note

    def get(self, request, *args, **kwargs):
        note = self.get_object()
        if kwargs.get('year'):
            # check date does match
            creation = self.get_object().creation_timestamp
            if (creation.year, creation.month, creation.day) != (
                int(kwargs['year']),
                int(kwargs['month']),
                int(kwargs['day']),
            ):
                raise Http404()
        if not note.published and not request.user.is_staff:
            raise PermissionDenied()
        return super(NoteView, self).get(request, *args, **kwargs)


class NoteEditView(UpdateView):
    model = Note
    fields = ['title', 'slug', 'text', 'tags', 'published']


class NoteAddView(CreateView):
    model = Note
    fields = ['title', 'slug', 'text', 'tags', 'published']


class NoteDeleteView(DeleteView):
    model = Note

    def get_success_url(self):
        return '/'


class HomeView(TemplateView):
    template_name = 'phyll/home.html'


class ListOnTagView(ListView):
    model = Note

    def get_queryset(self):
        qs = Note.objects.filter(tags__name__in=[self.kwargs['tag']])
        if not self.request.user.is_staff:
            qs = qs.filter(published=True)
        return qs
